//
// from http://www.sparkfun.com/tutorial/Nokia%206100%20LCD%20Display%20Driver.pdf
//

// ************************************************************************************************
//                LCDClearScreen.c
//
//     Clears the LCD screen to single color (BLACK)
//
//          Inputs:        none
//
//       Author: James P Lynch       August 30, 2007
// *************************************************************************************************
void LCDClearScreen(int color);

// *************************************************************************************
//                         LCDSetPixel.c
//
//       Lights a single pixel in the specified color at the specified x and y addresses
//
//       Inputs:    x     =   row address (0 .. 131)
//                  y     =   column address (0 .. 131)
//                  color =   12-bit color value rrrrggggbbbb
//                                  rrrr = 1111 full red
//                                           :
//                                  0000 red is off
//
//                                  gggg = 1111 full green
//                                           :
//                                  0000 green is off
//
//                                  bbbb = 1111 full blue
//                                           :
//                                  0000 blue is off
//
//          Returns:   nothing
//
//                Note: see lcd.h for some sample color settings
//
//       Author: James P Lynch       August 30, 2007
// *************************************************************************************
void LCDSetPixel(int x, int y, int color);

// *************************************************************************************************
//                         LCDSetLine.c
//
//       Draws a line in the specified color from (x0,y0) to (x1,y1)
//
//       Inputs:    x     =   row address (0 .. 131)
//                  y     =   column address (0 .. 131)
//                  color =         12-bit color value rrrrggggbbbb
//                                  rrrr = 1111 full red
//                                           :
//                                  0000 red is off
//
//                                  gggg = 1111 full green
//                                           :
//                                  0000 green is off
//
//                                  bbbb = 1111 full blue
//                                           :
//                                  0000 blue is off
//
//       Returns:   nothing
//
//       Note: good write-up on this algorithm in Wikipedia (search for Bresenham's line algorithm)
//             see lcd.h for some sample color settings
//
//       Authors:          Dr. Leonard McMillan, Associate Professor UNC
//                         Jack Bresenham IBM, Winthrop University (Father of this algorithm, 1962)
//
//       Note: taken verbatim from Professor McMillan's presentation:
//             http://www.cs.unc.edu/~mcmillan/comp136/Lecture6/Lines.html
//
// *************************************************************************************************
void LCDSetLine(int x0, int y0, int x1, int y1, int color);

// *****************************************************************************************
//                        LCDSetRect.c
//
//      Draws a rectangle in the specified color from (x1,y1) to (x2,y2)
//      Rectangle can be filled with a color if desired
//
//      Inputs:    x     =   row address (0 .. 131)
//                 y     =   column address (0 .. 131)
//                 fill  =   0=no fill, 1-fill entire rectangle
//                 color =   12-bit color value for lines     rrrrggggbbbb
//                                 rrrr = 1111 full red
//                                          :
//                                 0000 red is off
//
//                                 gggg = 1111 full green
//                                          :
//                                 0000 green is off
//
//                                 bbbb = 1111 full blue
//                                          :
//                                 0000 blue is off
//
//         Returns:   nothing
//
//      Notes:
//
//      The best way to fill a rectangle is to take advantage of the "wrap-around" featute
//      built into the Epson S1D15G00 controller. By defining a drawing box, the memory can
//      be simply filled by successive memory writes until all pixels have been illuminated.
//
//      1.  Given the coordinates of two opposing corners (x0, y0) (x1, y1)
//          calculate the minimums and maximums of the coordinates
//
//               xmin = (x0 <= x1) ? x0 : x1;
//               xmax = (x0 > x1) ? x0 : x1;
//               ymin = (y0 <= y1) ? y0 : y1;
//               ymax = (y0 > y1) ? y0 : y1;
//
//      2. Now set up the drawing box to be the desired rectangle
//
//               WriteSpiCommand(PASET);             // set the row boundaries
//               WriteSpiData(xmin);
//               WriteSpiData(xmax);
//               WriteSpiCommand(CASET);             // set the column boundaries
//               WriteSpiData(ymin);
//               WriteSpiData(ymax);
//
//       3. Calculate the number of pixels to be written divided by 2
//
//                NumPixels = ((((xmax - xmin + 1) * (ymax - ymin + 1)) / 2) + 1)
//
//                You may notice that I added one pixel to the formula.
//                This covers the case where the number of pixels is odd and we
//                would lose one pixel due to rounding error. In the case of
//                odd pixels, the number of pixels is exact.
//                in the case of even pixels, we have one more pixel than
//                needed, but it cannot be displayed because it is outside
//                the drawing box.
//
//                We divide by 2 because two pixels are represented by three bytes.
//                So we work through the rectangle two pixels at a time.
//
//       4.  Now a simple memory write loop will fill the rectangle
//
//                for (i = 0; i < ((((xmax - xmin + 1) * (ymax - ymin + 1)) / 2) + 1); i++) {
//                         WriteSpiData((color >> 4) & 0xFF);
//                         WriteSpiData(((color & 0xF) << 4) | ((color >> 8) & 0xF));
//                         WriteSpiData(color & 0xFF);
//                }
//
//
//       In the case of an unfilled rectangle, drawing four lines with the Bresenham line
//       drawing algorithm is reasonably efficient.
//
//
//       Author: James P Lynch        August 30, 2007
//  *****************************************************************************************
void LCDSetRect(int x0, int y0, int x1, int y1, unsigned char fill, int color);


//  *************************************************************************************
//                         LCDSetCircle.c
//
//       Draws a line in the specified color at center (x0,y0) with radius
//
//       Inputs:    x0     =    row address (0 .. 131)
//                  y0     =    column address (0 .. 131)
//                  radius =  radius in pixels
//                  color  =        12-bit color value rrrrggggbbbb
//
//       Returns:   nothing
//
//       Author: Jack Bresenham IBM, Winthrop University (Father of this algorithm, 1962)
//
//       Note: taken verbatim Wikipedia article on Bresenham's line algorithm
//             http://www.wikipedia.org
//
//  *************************************************************************************
void LCDSetCircle(int x0, int y0, int radius, int color);

// *****************************************************************************
//                       LCDPutChar.c
//
//    Draws an ASCII character at the specified (x,y) address and color
//
//         Inputs:          c      =   character to be displayed
//                          x      =   row address (0 .. 131)
//                          y      =   column address (0 .. 131)
//                          size   =   font pitch (SMALL, MEDIUM, LARGE)
//                         fcolor  =    12-bit foreground color value       rrrrggggbbbb
//                         bcolor =   12-bit background color value         rrrrggggbbbb
//
//         Returns:   nothing
//
// Notes:  Here's an example to display "E" at address (20,20)
//
//                       LCDPutChar('E', 20, 20, MEDIUM, WHITE, BLACK);
//
//                                       (27,20)       (27,27)
//                                          |             |
//                                          |             |
//                                     ^    V             V
//                                     : _ # #   # # # # #   0x7F
//                                     : _ _ #   # _ _ _ #   0x31
//                                     : _ _ #   # _ # _ _   0x34
//                                     x _ _ #   # # # _ _   0x3C
//                                     : _ _ #   # _ # _ _   0x34
//                                     : _ _ #   # _ _ _ #   0x31
//                                     : _ # #   # # # # #   0x7F
//                                     : _ _ _   _ _ _ _ _   0x00
//                                          ^             ^
//                                          |             |
//                                          |             |
//                                     (20,20)        (20,27)
//
//                                         ------y----------->
//
//
//                The most efficient way to display a character is to make use of the "wrap-around" feature
//                of the Epson S1D16G00 LCD controller chip.
//
//                Assume that we position the character at (20, 20) that's a (row, col) specification.
//                With the row and column address set commands, you can specify an 8x8 box for the SMALL and MEDIUM
//                characters or a 16x8 box for the LARGE characters.
//
//                         WriteSpiCommand(PASET);           // set the row drawing limits
//                         WriteSpiData(20);                 //
//                         WriteSpiData(27);                 // limit rows to (20, 27)
//
//                         WriteSpiCommand(CASET);           // set the column drawing limits
//                         WriteSpiData(20);                 //
//                         WriteSpiData(27);                 // limit columns to (20,27)
//
//                When the algorithm completes col 27, the column address wraps back to 20
//                At the same time, the row address increases by one (this is done by the controller)
//
//                We walk through each row, two pixels at a time. The purpose is to create three
//                data bytes representing these two pixels in the following format
//
//                         Data for pixel 0: RRRRGGGGBBBB
//                         Data for Pixel 1: RRRRGGGGBBBB
//
//                         WriteSpiCommand(RAMWR);           // start a memory write (96 data bytes to follow)
//
//                         WriteSpiData(RRRRGGGG);           // first pixel, red and green data
//                         WriteSpiData(BBBBRRRR);           // first pixel, blue data; second pixel, red data
//                         WriteSpiData(GGGGBBBB);           // second pixel, green and blue data
//                                  :
//       and so on until all pixels displayed!
//                                  :
//                         WriteSpiCommand(NOP);             // this will terminate the RAMWR command
//
//
//       Author: James P Lynch      August 30, 2007
// *****************************************************************************
void LCDPutChar(char c, int x, int y, int size, int fColor, int bColor);

// *************************************************************************************************
//                        LCDPutStr.c
//
//     Draws a null-terminates character string at the specified (x,y) address, size and color
//
//          Inputs:          pString =   pointer to character string to be displayed
//                           x       =   row address (0 .. 131)
//                           y       =   column address (0 .. 131)
//                           Size    =   font pitch (SMALL, MEDIUM, LARGE)
//                          fColor    =   12-bit foreground color value    rrrrggggbbbb
//                          bColor    =   12-bit background color value    rrrrggggbbbb
//
//
//          Returns:   nothing
//
//    Notes: Here's an example to display "Hello World!" at address (20,20)
//
//             LCDPutChar("Hello World!", 20, 20, LARGE, WHITE, BLACK);
//
//
//       Author: James P Lynch      August 30, 2007
// *************************************************************************************************
void LCDPutStr(char *pString, int x, int y, int Size, int fColor, int bColor);

